#include "app.h"
#include "lng.h"
#include "background.h"
#include "lng.h"
				
static UINT16       status, item_select;
static WCHAR		icons_dir[256];
UIS_DIALOG_T        dialog; 				// Диалог
ldrElf       	 	elf;					// Структура эльфа
DRAWING_BUFFER_T 	bufd = {NULL, 176, 220};// Буфер для рисования, Canvas.
	
/* Название приложения. Длина строки именно такая и никакая иначе */
const char app_name[APP_NAME_LEN] = ELF_NAME; 

/* Обработчики событий для каждого state-а */
/* Обработчики событий для APP_STATE_ANY (используется в любом state) */
static EVENT_HANDLER_ENTRY_T any_state_handlers[] =
{
	{ EV_PM_API_EXIT,				 HandleReqExit },	// PM API: cmd EXIT
    { STATE_HANDLERS_END,           		  NULL },
};

static EVENT_HANDLER_ENTRY_T main_state_handlers[] =
{
    { EV_INK_KEY_PRESS,               ListKeypress },
    { STATE_HANDLERS_END,                     NULL },
};

/* Таблица соответствий обработчиков, состояний и функций входа-выхода из состояния.
   Порядок состояний такой же, как в enum-e */
static const STATE_HANDLERS_ENTRY_T state_handling_table[] =
{
    { APP_STATE_ANY,              // State
      NULL,                       // Обработчик входа в state
      NULL,                       // Обработчик выхода из state
      any_state_handlers          // Список обработчиков событий
    },
    { APP_STATE_MAIN,
      MainStateEnter,
      StateExit,
      main_state_handlers
    },
	{ APP_STATE_BACKGROUND,
      BackgroundStateEnter,
	  BackgroundStateExit,
      background_state_handlers
    }
	
};

static const APP_STATE_NAME_TABLE_T state_name_table[] =
{
	{APP_STATE_ANY,			"ANY"},
	{APP_STATE_MAIN,		"MAIN"},
	{APP_STATE_BACKGROUND,	"BACKGROUND"},
	{APP_STATE_MAX,			NULL}
};

/* Это entry для всех эльфов, должна быть объявлена именно так */
/* uri - путь к эльфу (аналогично argv[0]) */
/* params - параметры эльфа (аналогично остальным argv) */
ldrElf * _start( WCHAR * uri, WCHAR * params )
{
    ldrElf *	pelf;
	UINT32		reserve;
	int			i;
	
	dbg("\n_start: Enter");
	dbg("_start: Enter");
	
		// ищем запущеный эльф по имени эльфа
	pelf = ldrFindElf(app_name);
	if ( pelf != NULL ) {
		// приложение уже запущено, вызвать запущеное
		ldrSendEvent( pelf->evbase );
		return NULL;
	}
	// база ивентов
	reserve = ldrRequestEventBase();
	// заполняем структуру эльфа
	elf.name = (char *)app_name;
	elf.app = NULL;
	
	// инитим таблицы ивентов
	for ( i=0; i<APP_STATE_MAX; i++ )
		reserve = ldrInitEventHandlersTbl((EVENT_HANDLER_ENTRY_T *)state_handling_table[i].htable, reserve);
	
	i = u_strlen( uri ) - 1;
	while ( i >= 0 && uri[i] != L'/' ) i--;
	i++;
	elf.dir = malloc( i * sizeof(WCHAR) );
	if ( elf.dir )
		u_strncpy( elf.dir, uri, i );

	u_strcpy(icons_dir,  L"/b/Elf2/elficons/" );
	
	elf.evbase = reserve ++;
	
	/* Регистрируем приложение */
	 status = APP_Register( &elf.evbase,            // На какое событие должно запускаться приложение
                           1,                       // Кол-во событий, на которое должно запускаться приложение
                           state_handling_table,    // Таблица состояний
                           APP_STATE_MAX,            // Количество состояний
                           (void*)startApp );   // Функция, вызываемая при запуске
	
	if ( status == RESULT_OK )
		ldrSendEvent( elf.evbase );
	 else {
		cprint(DEBUG_APP_NAME ":\x84 can't register app\n");
		return NULL;
	}

#ifdef DEBUG
	ReportPhoneInfo();
#endif

    return &elf; 
}

/* Функция вызываемая при старте приложения */
UINT32 startApp( EVENT_STACK_T *ev_st,  REG_ID_T reg_id,  UINT32 param2 )
{
	APP_ELF_T *			papp;
	LIST 				___; 
	List_Init(&___, NULL);
	
	
	if( AFW_InquireRoutingStackByRegId( reg_id ) == RESULT_OK ) {
		appShow(ev_st, elf.app );
		return RESULT_FAIL;
	}
	
	cprint( ELF_NAME ":\x82 Started\n" );
	AHG_Init();
	
	
    papp = (APP_ELF_T*)APP_InitAppData((void*)APPHandleEvent, 
                                            sizeof(APP_ELF_T), // Размер структуры приложения
                                            reg_id,
                                            0,
											0,			// history size
											1,			// priority
											AFW_APP_CENTRICITY_SECONDARY,
											AFW_FOCUS,
											AFW_POSITION_TOP );	
	
	if ( !papp )
	{
		dbg("can't init app data");
		ldrUnloadElf();
		return RESULT_FAIL;
	}
	
    status = APP_Start( ev_st,
                        &papp->apt,
                        APP_STATE_MAIN, // Начальное состояние
                        state_handling_table,
                        destroyApp,
                        app_name,
                        state_name_table );
	
	// Теперь приложение может обрабатывать события
	if ( status == RESULT_OK ) {
		elf.app = &papp->apt;
	}
	
    return RESULT_OK;
}


/* Функция выхода из приложения */
UINT32 destroyApp( EVENT_STACK_T *ev_st,  APPLICATION_T *app )
{
	EVENT_T *		event;

	event = AFW_GetEv(ev_st);
	dbgf("destroyApp: Enter, event = 0x%p, code = 0x%X", event, event->code);
	
	if ( event != NULL ) {
	
		// это нажатие красной кнопки
		appHide(ev_st, app);
		return RESULT_OK;
	}
	
	cprint( ELF_NAME ":\x8E Exit\n" );
	
	AHG_Done();

    /* Завершаем работу приложения */
    status = APP_ExitStateAndApp( ev_st, app, 0 );
	
    ldrUnloadElf();
   
    return status;
}

// Изменить стэйт + насильно показать
UINT32 AppChangeState( APPLICATION_T * app, UINT8 new_state, BOOL force_show )
{
	ADD_EVENT_DATA_T	ed;
	UINT32				ev_code;
	
	dbgf("AppChangeState: state = %s / %d",  state_name_table[new_state].name, force_show);
	
	ed.data_tag = 0;
	ed.data.pad[0] = new_state;
	ed.data.pad[1] = force_show;
	
	ev_code = ldrFindEventHandlerTbl(any_state_handlers, HandleChangeState);
	if ( ev_code > 0 )
		return
		AFW_CreateInternalQueuedEvPriv( ev_code, 0,
										app->afw_id,
										0, 0,
										&ed,
										FBF_FREE, 0, NULL, 0 );
	else
		return RESULT_FAIL;
}
UINT32 HandleChangeState( EVENT_STACK_T * ev_st, APPLICATION_T * app )
{
	APP_ELF_T *		papp = (APP_ELF_T *)(app);
	UINT32			status;
	EVENT_T *		event;
	
	status = RESULT_OK;
	event = AFW_GetEv(ev_st);
	dbgf("HandleChangeState: Enter, state = %s / %d", state_name_table[event->data.pad[0]].name, event->data.pad[1]);
	APP_ConsumeEv(ev_st, app);
	if ( app->state == APP_STATE_BACKGROUND && event->data.pad[0] != APP_STATE_BACKGROUND ) {
		// если уже в бэке
		papp->prev_state = event->data.pad[0];
		if ( event->data.pad[1] == TRUE ) {
			// нужно вызвать из бэка
			status = APP_UtilChangeState(papp->prev_state, ev_st, app);
		}
	}
	else if ( event->data.pad[0] == APP_STATE_BACKGROUND ) {
		// отправить в бэк
		if ( app->state != APP_STATE_BACKGROUND ) {
			papp->prev_state = app->state;
			status = APP_UtilChangeState(APP_STATE_BACKGROUND, ev_st, app);
		}
	}
	else {
		// просто изменить стэйт
		status = APP_UtilChangeState(event->data.pad[0], ev_st, app);
	}
	return status;
}
/* Обработчик входа в main state */
UINT32 MainStateEnter( EVENT_STACK_T *ev_st,  APPLICATION_T *app,  ENTER_STATE_TYPE_T type )
{
	APP_ELF_T *		papp = (APP_ELF_T *)(app);
	dbg("MainStateEnter: start");
	if(type != ENTER_STATE_ENTER) return RESULT_OK;
	
	app->dialog = UIS_CreateColorCanvas( &app->port, &bufd, true ); // создаём канвас
	
	
	papp->entries_num = ElfGetCount();
	DrawList(app);

	dbg("MainStateEnter: end");
	
    return RESULT_OK;
}

UINT32 StateExit( EVENT_STACK_T *ev_st,  APPLICATION_T *app,  EXIT_STATE_TYPE_T type )
{
	dialog = app->dialog;
	status = APP_UtilUISDialogDelete( &app->dialog );

	return status;
}

UINT32 HandleReqExit( EVENT_STACK_T *ev_st,  APPLICATION_T *app )
{
	dbg("HandleReqExit");
	cprint("\0x82"ELF_NAME": HandleReqExit\n");
	
	APP_ConsumeEv( ev_st, app );
	app->exit_status = TRUE;
	
	return RESULT_OK;
}

UINT32 ListKeypress( EVENT_STACK_T * ev_st,  APPLICATION_T *app )
{
    APP_ELF_T        *papp = (APP_ELF_T*)app;
	EVENT_T 		 *event = AFW_GetEv(ev_st);
	APP_ConsumeEv( ev_st, app );
	
    switch (event->data.key_pressed)
    {
		 case KEY_2:
	     case KEY_UP:
			 		if (item_select == 0)
					    item_select = papp->entries_num;
					if (item_select > 0) 
                        item_select--;
                    DrawList(app);
                    break;
					  
		 case KEY_8:	  
         case KEY_DOWN:
                    if (item_select < papp->entries_num) 
                        item_select ++;
					if (item_select == papp->entries_num)
						item_select = 0;
                    DrawList(app);
					break;

		//Свернуть
		case KEY_RED:
		case KEY_GREEN:
				appHide(ev_st,app );
				break;	
				

		//Выбрать
		case KEY_JOY_OK:
				ElfRestore( item_select );
				appHide(ev_st,app );
				break;
				
		case KEY_CLEAR:
		case KEY_SOFT_RIGHT:
				ElfUnLoad( item_select );
				return APP_UtilChangeState( APP_STATE_MAIN, ev_st, app );	
					
	}	
	return RESULT_OK;
}
//рисует список
UINT32 DrawList(APPLICATION_T *app)
{
	APP_ELF_T			*papp = (APP_ELF_T*)app;
	ldrElf				*pelf;
	AHIRECT_T		 	rect;
	WCHAR				icon_uri[256], name[32];
	
	UINT32			  	i = 0;

	SetBrushColor(0x000000);
	
	FillRect(10, 10, 40, 40*papp->entries_num); // Фон
	
	for(i = 0; i < papp->entries_num; i++) {
	
		pelf = ElfGet( i );
		u_mkpath( icon_uri, icons_dir, u_atou(pelf->name, name), 0 );
		
		if( !DL_FsFFileExist(icon_uri) )
			u_mkpath( icon_uri, icons_dir,  L"default.bmp", 0 );
			
		rect.x1 = 14; 
		rect.y1 = 14 + (i * 32); 
		rect.x2 = rect.x1 + 32; 
		rect.y2 = rect.y1 + 32;
		
		DrawBmpImageFS( icon_uri,  &rect, i*32, 0 );
	}

	//курсор
	SetBrushColor(0xFFFFFF); 
	DrawRect(13, 13 + (item_select * 32), 33, 33);
	
	
	AHG_Flush();
	
    return RESULT_OK;
}

UINT32 	ElfGetCount(  )
{
	UINT32					i;
	ldrElf					*pr;
	
	pr = ldrGetElfsList(0);
	i = 0;
	
	while ( pr ) 
	{
		i++;
		pr = pr->next;
	}
	
	return i;
}

ldrElf* ElfGet( UINT32 index )
{
	UINT32					i;
	ldrElf					*pr;
	
	pr = ldrGetElfsList(0);
	i = 0;
	
	while ( i == index ) 
	{
		i++;
		pr = pr->next;
	}
	
	if ( !pr ) return NULL;
	
	return pr;
}

UINT32 	ElfRestore( UINT32 index )
{
	UINT32					i;
	ldrElf					*pr;
	
	pr = ldrGetElfsList(0);
	i = 0;
	
	while ( i == index ) 
	{
		pr = pr->next;
		i++;
	}
	
	if ( !pr ) return RESULT_OK;
	
	
	if ( pr->app && AFW_InquireRoutingStackByRegId( pr->app->reg_id ) == RESULT_OK )
	{
		// эльф загружен и способен обрабатывать ивенты
		dbgf("UnLoadElf: name=%s, afwid=%x", pr->app->app_name, pr->app->afw_id );
		ldrSendEventToAppEmpty( pr->app->afw_id, pr->evbase );
	}
	
	return RESULT_OK;
}

UINT32 	ElfUnLoad( UINT32 index )
{
	UINT32					i;
	ldrElf					*pr;
	
	
	pr = ldrGetElfsList(0);
	i = 0;
	
	while ( i == index ) 
	{
		pr = pr->next;
		i++;
	}
	
	if ( !pr ) return RESULT_OK;
	
	
	if ( pr->app && AFW_InquireRoutingStackByRegId( pr->app->reg_id ) == RESULT_OK )
	{
		// эльф загружен и способен обрабатывать ивенты
		dbgf("UnLoadElf: name=%s, afwid=%x", pr->app->app_name, pr->app->afw_id );
		ldrSendEventToAppEmpty( pr->app->afw_id, ldrGetConstVal( EV_PM_API_EXIT ) );
	}
	else
	{
		// дохлый эльф - загружен но апп не запущено
		dbgf("UnLoadElf: addr=0x%x", pr->addr );
		ldrUnloadElfByAddr( pr->addr );
	}
	
	return RESULT_OK;
}

#ifdef	DEBUG
void ReportPhoneInfo()
{
	dbg("");
	
	dbgf("PhoneInfo: PhoneName = %s", ldrGetPhoneName());
	dbgf("PhoneInfo: PlatformName = %s", ldrGetPlatformName());
	dbgf("PhoneInfo: FirmwareVersion = %s", ldrGetFirmwareVersion());
	
	dbg("");
	const AHIDRVINFO_T	*ahi = ldrGetAhiDrvInfo();
	dbgf("PhoneInfo: ahi.drvName = %s", ahi->drvName);
	dbgf("PhoneInfo: ahi.drvVer = %s", ahi->drvVer);
	dbgf("PhoneInfo: ahi.totalMemory = %d b", ahi->totalMemory);
	dbgf("PhoneInfo: ahi.internalMemSize = %d b", ahi->internalMemSize);
	dbgf("PhoneInfo: ahi.externalMemSize = %d b", ahi->externalMemSize);
	dbg("");
}
#endif

